from api.miscellaneous import *
from api.objects.changelog import ChangelogEntry, SimpleChangelogEntrySubject
from api.objects.field import ObjectField


class ModificationEntry(ChangelogEntry):
    
    def __init__(self,
                 field: ObjectField,
                 object_id: int,
                 value_old_json: JsonTypes or None,
                 value_new_json: JsonTypes,
                 is_creation: bool = False):
        super().__init__(field, object_id)
        self.is_creation = is_creation
        self.value_old_json = value_old_json
        self.value_new_json = value_new_json
        
        if is_creation and value_old_json is not None:
            raise RuntimeError("At creation, old value must be none")  # pragma: no cover
    
    def add_parameters_to_api_json(self, entry_json: dict):
        super().add_parameters_to_api_json(entry_json)
        assert isinstance(self.subject, ObjectField)
        entry_json["type"] = "modification"
        entry_json["field_description"] = self.subject.get_description()
        if not self.is_creation:
            entry_json["old_value"] = self.value_old_json
        entry_json["new_value"] = self.value_new_json


class DeletionEntry(ChangelogEntry):
    
    def __init__(self, subject: SimpleChangelogEntrySubject, object_id: int, is_now_deleted: bool):
        super().__init__(subject, object_id)
        self.is_now_deleted = is_now_deleted
    
    def add_parameters_to_api_json(self, entry_json: dict):
        super().add_parameters_to_api_json(entry_json)
        entry_json["type"] = "deletion_change"
        entry_json["is_now_deleted"] = self.is_now_deleted


# Cannot import due to circular dependency
# noinspection PyUnresolvedReferences
class CreationEntry(ChangelogEntry):
    
    def __init__(self,
                 subject: SimpleChangelogEntrySubject,
                 object_id: int,
                 variant: str or None,
                 parent_type: "ObjectClass" or None,
                 parent_id: int or None):
        super().__init__(subject, object_id)
        self.variant = variant
        self.parent_type = parent_type
        self.parent_id = parent_id
        if (parent_type is None) != (parent_id is None):
            raise RuntimeError("parent_type and parent_id must be both None or both set")  # pragma: no cover
    
    def add_parameters_to_api_json(self, entry_json: dict):
        super().add_parameters_to_api_json(entry_json)
        entry_json["type"] = "creation"
        if self.variant is not None:
            entry_json["variant"] = self.variant
        if self.parent_type is not None:
            entry_json["parent_type"] = self.parent_type.api_id
            entry_json["parent_id"] = self.parent_id
    
    