import requests
from socket import gethostname

from api.routes import *


@api_route("/lecture_import/rwth_online/<int:course_id>", ["GET"])
@api_moderator_route(require_csrf_token=True)
def api_route_lecture_import_rwth_online(course_id: int):
    check_client_int(course_id, "URL.course_id")

    response = requests.get(
        url=f"https://online.rwth-aachen.de/RWTHonline/pl/ui/%24ctx/wbTermin_List.wbLehrveranstaltung?pStpSpNr={course_id}",
        timeout=30)
    if response.status_code != HTTP_200_OK:
        return {}, response.status_code
    
    from lxml import html
    document = html.fromstring(response.content.decode())
    
    data_list = [
        "pMode=T",
        "pInclPruef=N",
        "pInclPruefGepl=N",
        "pOutputFormat=99",
        "pCharset=UTF8",
        "pMaskAction=DOWNLOAD",
    ]
    for input_ele in document.xpath("//input[@name=\"pTerminNr\"]"):
        event_id = input_ele.value
        data_list.append(f"pTerminNr={event_id}")
    
    response = requests.post(
        url=f"https://online.rwth-aachen.de/RWTHonline/pl/ui/%24ctx/wbKalender.wbExport",
        data="&".join(data_list),
        timeout=30)
    mimetype = "text/calendar"
    if "Content-Type" in response.headers:
        mimetype = response.headers["Content-Type"]
    
    return ApiResponse(response.content, response.status_code, mimetype,
                       default_cache_control_max_age_sec=None)


# Not in API specification on purpose
@api_route("/diagnostics", ["GET"], allow_while_readonly=True)
@api_moderator_route()
def api_route_diagnostics():
    response = {
        "history": DIAGNOSTICS_TRACKER.get_history()
    }
    # noinspection PyBroadException
    try:
        response["host"] = gethostname()
    except Exception:
        pass
    return response
    
