from api.miscellaneous.constants import *

from api_test import ApiTest

# The python json parser does not unescape the escaped strings, so we need to put the extra \ here.
_TEST_DATA_FEEDBACK_1 = {
    "id": 1,
    "time_created": "2024-03-18T18:53:02",
    "text": "Hier FuNkToNiErT NICHTS!"
}
_TEST_DATA_FEEDBACK_2 = {
    "id": 2,
    "time_created": "2024-03-10T20:11:08",
    "email": "no-one@example.com",
    "text": "Ein bisschen Feedback"
}


class FeedbackTest(ApiTest):
    
    def test_get_feedback(self):
        self.do_json_request(
            "GET",
            "/feedback",
            expected_response_code=HTTP_401_UNAUTHORIZED
        )
        self.do_json_request(
            "GET",
            "/feedback",
            None,
            {
                "page_count": 1,
                "page": [
                    _TEST_DATA_FEEDBACK_1,
                    _TEST_DATA_FEEDBACK_2,
                ]
            },
            use_moderator_login=True
        )
        self.do_json_request(
            "GET",
            "/feedback?entries_per_page=10&page=0",
            None,
            {
                "page_count": 1,
                "page": [
                    _TEST_DATA_FEEDBACK_1,
                    _TEST_DATA_FEEDBACK_2,
                ],
            },
            use_moderator_login=True
        )
        self.do_json_request(
            "GET",
            "/feedback?entries_per_page=10&page=1",
            None,
            {
                "page_count": 1,
                "page": []
            },
            use_moderator_login=True
        )
    
    def test_put_feedback(self):
        self.do_json_request(
            "PUT",
            "/feedback/new",
            {
                "email": "",
                "text": "This is an invalid email"
            },
            expected_response_code=HTTP_400_BAD_REQUEST
        )
        self.do_json_request(
            "PUT",
            "/feedback/new",
            {
                "email": "something",
                "text": "This is an invalid email"
            },
            expected_response_code=HTTP_400_BAD_REQUEST
        )
        self.do_json_request(
            "PUT",
            "/feedback/new",
            {
                "email": "something@",
                "text": "This is an invalid email"
            },
            expected_response_code=HTTP_400_BAD_REQUEST
        )
        self.do_json_request(
            "PUT",
            "/feedback/new",
            {
                "email": "@still-wrong",
                "text": "This is an invalid email"
            },
            expected_response_code=HTTP_400_BAD_REQUEST
        )
        self.do_json_request(
            "PUT",
            "/feedback/new",
            {
                "email": "something@still-wrong",
                "text": "This is an invalid email"
            },
            expected_response_code=HTTP_400_BAD_REQUEST
        )
        self.do_json_request(
            "PUT",
            "/feedback/new",
            {
                "email": f"something@still-wrong.now-correctButWa{'a' * 100}ayTooLong",
                "text": "This is an invalid email"
            },
            expected_response_code=HTTP_400_BAD_REQUEST
        )
        self.do_json_request(
            "PUT",
            "/feedback/new",
            {
                "text": f"This text is soo{'o' * 20000} long"
            },
            expected_response_code=HTTP_400_BAD_REQUEST
        )
        self.do_json_request(
            "PUT",
            "/feedback/new",
            {
                "text": "Noch mehr feedback"
            },
            expected_response_code=HTTP_201_CREATED
        )
        self.do_json_request(
            "GET",
            "/feedback",
            None,
            {
                "page_count": 1,
                "page": [
                    {
                        "text": "Noch mehr feedback",
                    },
                    _TEST_DATA_FEEDBACK_1,
                    _TEST_DATA_FEEDBACK_2,
                ]
            },
            ignore_value_keys=["$.page[0].id", "$.page[0].time_created"],
            use_moderator_login=True
        )
        self.do_json_request(
            "PUT",
            "/feedback/new",
            {
                "email": "me@example.com",
                "text": "Geht so"
            },
            expected_response_code=HTTP_201_CREATED
        )
        self.do_json_request(
            "GET",
            "/feedback",
            None,
            {
                "page_count": 1,
                "page": [
                    {
                        "email": "me@example.com",
                        "text": "Geht so",
                    },
                    {
                        "text": "Noch mehr feedback",
                    },
                    _TEST_DATA_FEEDBACK_1,
                    _TEST_DATA_FEEDBACK_2,
                ]
            },
            ignore_value_keys=["$.page[0].id", "$.page[0].time_created", "$.page[1].id", "$.page[1].time_created"],
            use_moderator_login=True
        )
